/*********************************************************/
// MODULE:			ChidRegFile
//
// FILE NAME:		ChidRegFile.v
// VERSION:			1.1
// DATE:				Created Feb 9, 2003
//                Modified Nov 15, 2003 by Larry Merkle
//                - "Unregistered" ReadDataN to eliminate
//                  the clock cycle delay between changing 
//                  ReadRegN and seeing the effect on ReadDataN
//						Modified Oct 26, 2005 by Keith Kaffenberger
//						- To interface with the Mach5 processor
//						Modified 02 Nov 2005 by Guy Builta
//						- Fixed 0, 1 registers to be constant.
// AUTHOR:			Larry Merkle
//
// DESCRIPTION:	This module defines a register file with 
//                eight 16-bit registers, two read ports, 
//                and one write port.  
//
/*********************************************************/
module ChidRegFile(
   ReadReg1,
	ReadReg2,
	ReadReg3,
   WriteReg,
   WriteData,
   RegWrite,
   ReadData1,
	ReadData2,
	ReadData3,
	Reset,               
   CLK);

// Definitions

parameter NUM_REGS = 16;
`define ADDR_SZ 4
`define REG_WIDTH 16

// Inputs

input [`ADDR_SZ-1:0] ReadReg1;
input [`ADDR_SZ-1:0] ReadReg2;
input [`ADDR_SZ-1:0] ReadReg3;
input [`ADDR_SZ-1:0] WriteReg;
input [`REG_WIDTH-1:0] WriteData;
input RegWrite;
input CLK;
input Reset;

// Outputs

output [`REG_WIDTH-1:0] ReadData1;
output [`REG_WIDTH-1:0] ReadData2;
output [`REG_WIDTH-1:0] ReadData3;

// Signals

wire [`REG_WIDTH-1:0] ReadData1;
wire [`REG_WIDTH-1:0] ReadData2;
wire [`REG_WIDTH-1:0] ReadData3;

// The registers

reg [`REG_WIDTH-1:0] Reg [NUM_REGS-1:0];

assign ReadData1 = Reg[ ReadReg1 ];
assign ReadData2 = Reg[ ReadReg2 ];
assign ReadData3 = Reg[ ReadReg3 ];

// Operation

always @ ( posedge CLK ) begin

	Reg[ 0 ] <= 0;
	Reg[ 1 ] <= 1;
   if ( RegWrite && WriteReg != 0 && WriteReg != 1 )
      Reg[ WriteReg ] <= WriteData;
end

always @ (posedge Reset) begin
	Reg[ 0 ] <= 0;
	Reg[ 1 ] <= 0;
	Reg[ 2 ] <= 0;
	Reg[ 3 ] <= 0;
	Reg[ 4 ] <= 0;
	Reg[ 5 ] <= 0;
	Reg[ 6 ] <= 0;
	Reg[ 7 ] <= 0;
	Reg[ 8 ] <= 0;
	Reg[ 9 ] <= 0;
	Reg[ 10 ] <= 0;
	Reg[ 11 ] <= 0;
	Reg[ 12 ] <= 0;
	Reg[ 13 ] <= 0;
	Reg[ 14 ] <= 0;
	Reg[ 15 ] <= 16'h0500;
end
	

endmodule