/*********************************************************/
// MODULE:	SPRegFile
//
// FILE NAME:	SPRegFile.v
// VERSION:	1.1
// DATE:	Created Feb 9, 2003
//              Modified Nov 15, 2003 by Larry Merkle
//              - "Unregistered" ReadDataN to eliminate
//              the clock cycle delay between changing 
//              ReadRegN and seeing the effect on ReadDataN
//		Modifed 25 Oct 2005 by Guy Builta to support SP Reg Specification
// AUTHOR:			Larry Merkle
//
// DESCRIPTION:	This module defines a register file with 
//                eight 16-bit registers, two read ports, 
//                and one write port.  
//
/*********************************************************/
module SPRegFile(
   ReadReg1,
   WriteReg,
   WriteData,
   RegWrite,
   ReadData1,               
   Switches,
   Display0,
   Display1,
   Display2,
   Display3,
	Reset,
   CLK);

// Definitions

parameter NUM_REGS = 32;
`define ADDR_SZ 5
`define REG_WIDTH 16

// Inputs

input [`ADDR_SZ-1:0] ReadReg1;
input [`ADDR_SZ-1:0] WriteReg;
input [`REG_WIDTH-1:0] WriteData;
input [`REG_WIDTH-1:0] Switches;
input RegWrite;
input CLK;
input Reset;

// Outputs

output [`REG_WIDTH-1:0] ReadData1;
output [`REG_WIDTH-1:0] Display0;
output [`REG_WIDTH-1:0] Display1;
output [`REG_WIDTH-1:0] Display2;
output [`REG_WIDTH-1:0] Display3;

// Signals

wire [`REG_WIDTH-1:0] ReadData1;
wire [`REG_WIDTH-1:0] Display0;
wire [`REG_WIDTH-1:0] Display1;
wire [`REG_WIDTH-1:0] Display2;
wire [`REG_WIDTH-1:0] Display3;

// The registers

reg [`REG_WIDTH-1:0] Reg [NUM_REGS-1:0];

assign ReadData1 = Reg[ ReadReg1 ];
assign Display0 = Reg[ 0 ];
assign Display1 = Reg[ 1 ];
assign Display2 = Reg[ 2 ];
assign Display3 = Reg[ 3 ];

// Operation

always @ ( posedge CLK ) begin
   Reg[ 5 ] <= Switches;

   if ( RegWrite )
      Reg[ WriteReg ] <= WriteData;
end

always @ (posedge Reset) begin
	Reg[ 0 ] <= 0;
	Reg[ 1 ] <= 0;
	Reg[ 2 ] <= 0;
	Reg[ 3 ] <= 0;
	Reg[ 4 ] <= 0;
	Reg[ 5 ] <= 0;
	Reg[ 6 ] <= 0;
	Reg[ 7 ] <= 0;
	Reg[ 8 ] <= 0;
	Reg[ 9 ] <= 0;
	Reg[ 10 ] <= 0;
	Reg[ 11 ] <= 0;
	Reg[ 12 ] <= 0;
	Reg[ 13 ] <= 0;
	Reg[ 14 ] <= 0;
	Reg[ 15 ] <= 0;
	Reg[ 16 ] <= 0;
	Reg[ 17 ] <= 0;
	Reg[ 18 ] <= 0;
	Reg[ 19 ] <= 0;
	Reg[ 20 ] <= 0;
	Reg[ 21 ] <= 0;
	Reg[ 22 ] <= 0;
	Reg[ 23 ] <= 0;
	Reg[ 24 ] <= 0;
	Reg[ 25 ] <= 0;
	Reg[ 26 ] <= 0;
	Reg[ 27 ] <= 0;
	Reg[ 28 ] <= 0;
	Reg[ 29 ] <= 0;
	Reg[ 30 ] <= 0;
	Reg[ 31 ] <= 0;
end

endmodule
